/// pdrファイル読み込み、表示
/**
 * pdrファイル読込、表示
 * ParaFla!っていうフリーのflashのプログラム生成ツール用の
 * ParaDraw!のデータを読み込みSDLのOpenGLで表示します。
 * See_Also:
 *   $(LINK2 http://www.geocities.jp/coa9999/,ParaFla!)
 */

private import std.string;
private import std.file;
private import opengl;

/// pdrファイル読込
/**
 * pdrファイルを読み込んでfloat[][]で返します。
 * Params:
 * name = ファイル名
 * path = pdrファイルの入っているディレクトリ名
 * Returns: pdrデータが入っているfloat２次元配列
 * 1次元の配列データには0,1,2,3番目にポリゴンの色、4,5,6,7番目に線の色、8番目以降にポリゴンデータが入っています。
 */
float[][] getChar(char[] name, char[] path = ".") {
	float out1[][];
	
	char[] dt = cast(char[])read(path ~ "/" ~ name);
	dt = dt.replace("\r\n", "\n").replace("\r", "\n");
	char[][] line = dt.split("\n");
	int numPath = atoi(line[3]);
	int centerX = atoi(line[1].split(",")[0])*10;
	int centerY = atoi(line[1].split(",")[1])*10;
	int lno = 4;
	for (int i = 0; i < numPath; i++) {
		char[][] pathHead = line[lno++].split(",");
		int numAnc = atoi(pathHead[0])+1;
		int lineColor = atoi(pathHead[4]);
		int fillColor = atoi(pathHead[6]);
		float output[];
		output.length = 8;
		output[0] =  (fillColor      & 0xff) / 255.0;
		output[1] = ((fillColor>>8)  & 0xff) / 255.0;
		output[2] = ((fillColor>>16) & 0xff) / 255.0;
		output[3] = ((fillColor>>24) & 0xff) / 255.0;
		output[4] =  (lineColor      & 0xff) / 255.0;
		output[5] = ((lineColor>>8 ) & 0xff) / 255.0;
		output[6] = ((lineColor>>16) & 0xff) / 255.0;
		output[7] = ((lineColor>>24) & 0xff) / 255.0;
		if (line[0] == "PDR030")lno++;
		float output2[];
		output2.length = numAnc * 3;
		for (int j = 0; j < numAnc; j++) {
			char[][] s = line[lno++].split(",");
			int x = atoi(s[1]);
			int y = atoi(s[2]);
			output2[j*3    ] = x - centerX;
			output2[j*3 + 1] = y - centerY;
			output2[j*3 + 2] = 0;
		}

		// 反転チェック
		V v1 = V(output2[ 3], output2[ 4]);
		V v2 = V(output2[ 0], output2[ 1]);
		V v3 = V(output2[ 6], output2[ 7]);
		float s = (v2-v1) * (v3-v1);
		if (s > 0) {
			float tmp;
			// 4 * 3 = 12 12 - 2 = 10 / 2 = 5 だ。 i1 = 0のとき。 0 と 9 1,10, 2,11
			// i1 = 3 のとき、3
			float[] output3;
			for (int i1 = output2.length; i1 > 0; i1-=3) {
				output3 ~= output2[i1-3 .. i1];
			}
			output2 = output3;
		}
		output ~= output2;
		out1.length = out1.length + 1;
		out1[out1.length - 1] = output;
	}
	return out1;
}

/// pdr描画
/**
 * pdrファイルから読み込んだデータを描画します。
 * params:
 * dt = pdrデータの入ったfloat[][]配列
 */
void draw(float[][] dt) {
	foreach(float[] d; dt) {
		draw(d);
	}
}

/// pdr1ポリゴン描画
/**
 * pdrファイルから読み込んだデータの1ポリゴンを描画します。
 * params:
 * d = pdrデータの入ったfloat[]配列
 * 1次元の配列データには0,1,2,3番目にポリゴンの色、4,5,6,7番目に線の色、8番目以降にポリゴンデータが入っています。
 */
void draw(float[] d) {
	if (d[3] != 0.0f) {
		glColor4f(d[0], d[1], d[2], d[3]);
		glBegin(GL_TRIANGLE_FAN);
		for (int i = 0; i < (d.length - 8)/3; i++) {
			glVertex3f  (d[8+i*3], d[8+i*3+1], d[8+i*3+2]);
		}
		glEnd();
	}
	glColor4f(d[4], d[5], d[6], d[7]);

	glBegin(GL_LINE_LOOP);
	for (int i = 0; i < (d.length - 8)/3; i++)
		glVertex3f  (d[8+i*3], d[8+i*3+1], d[8+i*3+2]);

	glEnd();
}

/// pdrデータ複数読み込み
/**
 * pdrファイル名が1行に1つずつ入っているテキストファイルを読み込み
 * pdrファイルを複数一度に読み込みます。
 * Params:
 * name = pdrファイルリストテキストファイル名
 * path = pdrファイルの入っているディレクトリ名
 * Returns: pdrデータが複数入っているfloat３次元配列
 */
float[][][] getAllChar(char[] fname, char[] path=".") {
	float[][][] dataArray;
	foreach (char[] name; (cast(char[])read(path ~ "/" ~ fname)).replace("\r\n","\n").replace("\r","\n").split("\n")) {
		if (name == "")continue;
		dataArray.length = dataArray.length + 1;
		dataArray[dataArray.length - 1] = getChar(name, path);
	}
	return dataArray;
}

float[][][char[]] pdrs;
void loadDir(char[] dir) {
//	pdrs = null;
	foreach (char[] name; listdir(dir)) {
		try {
			pdrs[name] = pdr.getChar(name, dir);
		} catch (Exception e){}
	}
}

void draw(char[] name) {
	try {
		draw(pdrs[name]);
	} catch (Error e) {
		printf("pdr.draw error %.*s\n", name);
	}
}

int getLength(char[] name) {
	try {
		return pdrs[name].length;
	} catch (Error e) {
		return 0;
	}
}

void draw(char[] name, int no) {
	try {
		draw(pdrs[name][no]);
	} catch (Error e) {
		printf("pdr.draw error %.*s %d\n", name, no);
	}
}

import std.string;
struct V{
	float x;
	float y;
	V opAdd(V v) {
		V r;
		r.x = x + v.x;
		r.y = y + v.y;
		return r;
	}
	V opSub(V v) {
		V r;
		r.x = x - v.x;
		r.y = y - v.y;
		return r;
	}
	V normalize() {
		float l = std.math.sqrt(x * x + y * y);
		return V(x/l,y/l);
	}
	float abs() {
		return std.math.sqrt(x * x + y * y);
	}
	// 外積
	float opMul(V v) {
		return x * v.y - v.x * y;
	}
	float opMul(float vx, float vy) {
		return x * vy - vx * y;
	}
	V opMul(float n) {
		return V(x * n, y * n);
	}
	V opDiv(float n) {
		return V(x / n, y / n);
	}

	// 内積
	float opXor(V v) {
		return x * v.x + v.y * y;
	}
	float opXor(float vx, float vy) {
		return x * vy + vx * y;
	}
	int opCmp(float vx, float vy) {
		printf("cmp\n");
		if (vx==x && vy==y) return 0;
		return 1;
	}
	char[] toString() {
		return std.string.toString(x) ~ "," ~ std.string.toString(y);
	}
	static V opCall(){
		V s;
		s.x = s.y = 0;
		return s;
    }
	static V opCall(float u,float v){
		V s;
		s.x = u;
		s.y = v;
		return s;
    }
}

