private import all;

enum {
	GROUP_GFX,
	GROUP_BG,
	GROUP_SHIP,
	GROUP_GENERATOR,
	GROUP_TANK,
	GROUP_PARTICLE,
	GROUP_SHOT,
	GROUP_BULLET,
	GROUP_BULLET2,
	GROUP_CAMERA2,
	GROUP_CURSOR,
	GROUP_TEXT,
	GROUP_GAME,
	GROUP_MAX,
};

class TaskList {
	private static int maxSize;
	private static int maxGroup;
	private static int recordSize;
	private byte* nodes;
	private int*[] nodes2;
	private int[] index;
	private int[] last;
	public int[] groupCount;
	private int empty;

	private struct NODE {
		int prev;
		int next;
	}

	private static this() {
		maxSize = 0;
		maxGroup = 0;
	}

	static void addSize(int size, int group) {
		maxSize = max(maxSize, size);
		maxGroup = max(maxGroup, group);
	}

	this(int size) {
		exit = false;
		empty = 0;
		index.length = last.length = groupCount.length = maxGroup + 1;
		
		for(int i = 0; i < index.length; i++) {
			index[i] = -1;
			last[i] = -1;
		}
		recordSize = ((maxSize + NODE.sizeof + 3)/4 * 4);
		printf("maxGroup = %d maxSize = %d recordSize=%d\n", maxGroup, maxSize, recordSize);
		nodes2.length = (recordSize / 4) * size;
		nodes = cast(byte*)nodes2;
		printf("task list memory size %d bytes\n", recordSize * size);
		for (int i = 0; i < size; i++) {
			getNODE(i).prev = i - 1;
			getNODE(i).next = i + 1;
		}
		getNODE(size - 1).next = -1;
	}

	private NODE* getNODE(int no) {
		return (cast(NODE*)&nodes[no * recordSize]);
	}

	void* malloc(int size, int group) {
		if (empty != -1) {
			int newNo = empty;
			int next = getNODE(empty).next;
			if (index[group] == -1) {
				index[group] = empty;
			} else {
				getNODE(last[group]).next = empty;
			}
			getNODE(empty).prev = last[group];
			last[group] = empty;
			getNODE(empty).next = -1;
			empty = next;
			void* n = cast(void*)(&nodes[newNo * recordSize] + NODE.sizeof);
			return n;
		}
		return null;
	}

	void free(void* mem, int group) {
		int no = ((mem - cast(void*)nodes ) - NODE.sizeof) / recordSize;
		int next = getNODE(no).next;
		if (next != -1) {
			getNODE(next).prev = getNODE(no).prev;
		} else {
			last[group] = getNODE(no).prev;
		}
		int prev = getNODE(no).prev;
		if (prev != -1) {
			getNODE(prev).next = next;
		} else {
			index[group] = getNODE(no).next;
		}
		getNODE(no).next = empty;
		empty = no;
	}

	int opApply(int delegate(inout TaskObject) dg) {
		int result = 0;
		for (int i = 0; i < index.length; i++) {
			int no = index[i];
			while (no != -1) {
				void* n = cast(void*)(&nodes[no * recordSize] + NODE.sizeof);
				TaskObject node = cast(TaskObject)n;
				no = getNODE(no).next;
				result = dg(node);
				if (result)	break;
			}
		}
		return result;
	}

	void map(int group, int delegate(inout TaskObject) dg) {
		int result = 0;
		int no = index[group];
		while (no != -1) {
//			printf("%d\n", no);
			void* n = cast(void*)(&nodes[no * recordSize] + NODE.sizeof);
			TaskObject node = cast(TaskObject)n;
			no = getNODE(no).next;
			result = dg(node);
			if (result)	break;
		}
		return result;
	}

	void move() {
		for (int i = 0; i < index.length; i++) {
			int no = index[i];
			int count = 0;
			while (no != -1) {
				void* n = cast(void*)(&nodes[no * recordSize] + NODE.sizeof);
				TaskObject node = cast(TaskObject)n;
				no = getNODE(no).next;
				try {
					if (node.move)
						node.move();
				} catch (Object e) {
					printf("move error ");
					try {
						printf("%.*s %.*s x %f y %f\n",
							e.toString(),
							(cast(Object)node).classinfo.name, node.x, node.y);
					} catch (Object o) {
						printf("%.*s index = %d no = %d node is not TaskObject!\n", e.toString(), i, no);
					}
				}
				count++;
			}
			groupCount[i] = count;
//			if (i == GROUP_BULLET2)
//				printf("%d %d\n", i, count);
		}
	}

	void draw() {
		foreach(TaskObject o; this) {
			try {
				if (o.draw) o.draw();
			} catch (Object e) {
					printf("draw error ");
				try {
					printf("%.*s %.*s\n", 
						e.toString(), o.classinfo.name);
				} catch (Object o2) {
					printf("%.*s node is not TaskObject!\n", e.toString());
				}
			}
		}
/*			glBegin(GL_LINE_LOOP);
			glVertex3f  (0, 0, 0);
			glVertex3f  (100.0,100.0, 0);
			glEnd();
*/
			glPushMatrix();
			glTranslatef(20000.0, 20000.0, 0);
			glScalef( 0.1, 0.1, 1.0);
			glColor4f(1.0, 0.0, 0.0, 1.0);

			map(GROUP_TANK, delegate int(inout TaskObject o) {
				glBegin(GL_LINE_LOOP);
				glVertex3f  (o.x,o.y, 0);
				glVertex3f  (o.x+1000.0,o.y+1000.0, 0);
				glEnd();
				return 0;
			});
		try {
			map(GROUP_SHIP, delegate int(inout TaskObject o) {
				glBegin(GL_LINE_LOOP);
				glVertex3f  (o.x,o.y, 0);
				glVertex3f  (o.x+1000.0,o.y+1000.0, 0);
				glEnd();
				return 0;
			});
		} catch (Exception e){}
		glPopMatrix();
	}
	bool exit;
}

template GROUP(int g) {
	static this() {
		printf("addSize %.*s %d\n", typeof(this).classinfo.name, typeof(this).classinfo.init.length);
		TaskList.addSize(typeof(this).classinfo.init.length, g);
	}

	public new(size_t sz) {
//		if (g == GROUP_BULLET) printf("%d ", g);
		void* p = tl.malloc(sz, g);
		if (!p)
			throw new OutOfMemoryException();
		return p;
	}

	public delete(void* p) {
//		if (g == GROUP_BULLET) printf("delete %d %d\n", p, g);
		if (p) {
			tl.free(p, g);
		}
	}
	this() {
	}
}

TaskList tl;

class TaskObject {
	mixin GROUP!(0);
	public void delegate() move;
	public void delegate() draw;
	float x;
	float y;
	int collNo;
	float direction;
	float vX;
	float vY;
	int energy;
	int size;
	TaskObject target;

	this() {
		x = 0;
		y = 0;
		vX = 0;
		vY = 0;
		direction = 0;
	}

	void defaultMove() {
//		printf("default move %.*s\n", (cast(Object)this).classinfo.name);
	}
	static char[] getName(Object o) {
		char[] name = o.classinfo.name;
		int n = rfind(name, ".");
		if (n != -1) name = name[n+1..name.length];
		return name;
	}
	void defaultDraw() {
		glPushMatrix();
		glTranslatef(x, y, 0);
		glRotatef(direction, 0, 0, -1);
		pdr.draw(getName(this) ~ ".pdr");
		glPopMatrix();
	}

	void clear() {
		move = &deleteThis;
	}

	void deleteThis() {
		delete this;
	}
}
